@@ -21,6 +21,7 @@ class _Reactor(object):
     def __init__(self):
         self._protos = []
         self._handlers = []
+        self.on_stop = None
 
     def connect(self, factory, host, port, secure, conn=None):
         proto = factory.build_protocol(host, port, secure)
@@ -31,6 +32,7 @@ class _Reactor(object):
     def listen(self, factory, host='127.0.0.1', port=0):
         listener_factory = tcp_server.TCPServerHandler(factory, self)
         proto = listener_factory.build_protocol(host, port)
+        factory.listener_factory = listener_factory
         self._protos.append(proto)
         self._handlers.append(listener_factory)
         return proto.sockname()
@@ -62,6 +64,8 @@ class _Reactor(object):
         self._handlers = []
         msg.log('Reactor shut down.')
         editor.status_message('Disconnected.')
+        if self.on_stop:
+            self.on_stop()
 
     def is_ready(self):
         if not self._handlers:
@@ -87,7 +91,7 @@ class _Reactor(object):
         editor.call_timeouts()
 
     def block(self):
-        while True:
+        while self._protos or self._handlers:
             self.tick(.05)
 
     def select(self, timeout=0):
