@@ -8,8 +8,10 @@ AUTH = {}
 
 DEBUG = False
 SOCK_DEBUG = False
+SOCK_SINGLE_READ = False
 
 EXPERT_MODE = False
+FLOOTTY_SAFE = True
 
 ALERT_ON_MSG = True
 LOG_TO_CONSOLE = False
@@ -39,6 +41,7 @@ WORKSPACE_WINDOW = None
 
 PERMS = []
 FOLLOW_MODE = False
+FOLLOW_USERS = set()
 SPLIT_MODE = False
 
 AUTO_GENERATED_ACCOUNT = False
@@ -51,8 +54,6 @@ TICK_TIME = 100
 AGENT = None
 IGNORE = None
 
-IGNORE_MODIFIED_EVENTS = False
 VIEW_TO_HASH = {}
 
-FLOORC_PATH = os.path.expanduser(os.path.join('~', '.floorc'))
 FLOORC_JSON_PATH = os.path.expanduser(os.path.join('~', '.floorc.json'))
