@@ -28,22 +28,6 @@ except NameError:
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-# boilerplate to allow running as script directly
-if __name__ == "__main__" and __package__ is None:
-    import sys
-    import os
-    # The following assumes the script is in the top level of the package
-    # directory.  We use dirname() to help get the parent directory to add to
-    # sys.path, so that we can import the current package.  This is necessary
-    # since when invoked directly, the 'current' package is not automatically
-    # imported.
-    parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-    sys.path.insert(0, parent_dir)
-    import flootty
-    assert flootty
-    __package__ = str("flootty")
-    del sys, os
-
 import atexit
 import fcntl
 import json
@@ -63,6 +47,20 @@ import time
 import base64
 import collections
 
+# boilerplate to allow running as script directly
+if __name__ == "__main__" and __package__ is None:
+    # The following assumes the script is in the top level of the package
+    # directory.  We use dirname() to help get the parent directory to add to
+    # sys.path, so that we can import the current package.  This is necessary
+    # since when invoked directly, the 'current' package is not automatically
+    # imported.
+    parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+    sys.path.insert(0, parent_dir)
+    import flootty
+    assert flootty
+    __package__ = str("flootty")
+
+
 PY2 = sys.version_info < (3, 0)
 
 
@@ -74,12 +72,12 @@ except (ImportError, AttributeError):
 
 try:
     from . import version
-    from .floo.common import api, cert, shared as G, migrations, utils
+    from .floo.common import api, cert, shared as G, utils
     from .floo.common.exc_fmt import str_e
     assert api and G and utils and version
 except (ImportError, ValueError):
     import version
-    from floo.common import api, cert, shared as G, migrations, utils
+    from floo.common import api, cert, shared as G, utils
     from floo.common.exc_fmt import str_e
 
 
@@ -189,8 +187,6 @@ def get_now_editing_workspaces():
 
 
 def main():
-    if not os.path.exists(G.FLOORC_JSON_PATH):
-        migrations.migrate_floorc()
     utils.reload_settings()
     default_auth = G.AUTH.get(G.DEFAULT_HOST, {})
     parser = optparse.OptionParser(usage=usage)
@@ -258,6 +254,11 @@ def main():
                       action="store_true",
                       help="Resize your terminal to the host terminal size.")
 
+    parser.add_option("--shell",
+                      dest="shell",
+                      default=os.environ.get("SHELL", None),
+                      help="The shell you would like to use with flootty. Defaults to $SHELL.")
+
     parser.add_option("-P", "--preserve-ps1",
                       dest="set_prompt",
                       default=True,
@@ -315,6 +316,9 @@ def main():
         if not options.host:
             options.host = floo.get('host')
 
+    if options.host is None:
+        options.host = G.DEFAULT_HOST
+
     if options.host != G.DEFAULT_HOST and options.secret == default_auth.get('secret'):
         auth = G.AUTH.get(options.host)
         if not auth:
@@ -324,7 +328,7 @@ def main():
 
     if not options.workspace or not options.owner:
         try:
-            now_editing = api.get_now_editing_workspaces()
+            now_editing = get_now_editing_workspaces()
         except Exception as e:
             print(str_e(e))
         else:
@@ -337,11 +341,17 @@ def main():
         if len(term_name) != 0:
             die("I don't understand why you gave me a positional argument.")
 
-    for opt in ['workspace', 'owner', 'username', 'secret']:
+    for opt in ['username', 'secret']:
+        if not getattr(options, opt):
+            parser.error('%s not given. Please use --%s or add credentials to ~/.floorc.json' % (opt, opt))
+
+    for opt in ['workspace', 'owner']:
         if not getattr(options, opt):
             parser.error('%s not given' % opt)
 
     color_reset = '\033[0m'
+    if not G.FLOOTTY_SAFE:
+        options.safe = G.FLOOTTY_SAFE
     if options.safe:
         green = '\033[92m'
         print('%sTerminal is safe. Other users will not be able to send [enter]%s' % (green, color_reset))
@@ -357,7 +367,8 @@ def main():
 
 
 def walk_up(path):
-    step_up = lambda x: os.path.realpath(os.path.join(x, '..'))
+    def step_up(p):
+        return os.path.realpath(os.path.join(p, '..'))
     parent = step_up(path)
     while parent != path:
         yield path
@@ -616,28 +627,32 @@ class Flootty(object):
     def on_create_term(self, data):
         if data.get('term_name') != self.term_name:
             return
-        self.term_id = data.get('id')
+        self.term_id = int(data.get('id'))
         self.create_term()
 
     def on_delete_term(self, data):
-        if data.get('id') != self.term_id:
+        term_id = int(data.get('id'))
+        if term_id != self.term_id:
             return
         die('User %s killed the terminal. Exiting.' % (data.get('username')))
 
     def on_update_term(self, data):
-        if data.get('id') != self.term_id:
+        term_id = int(data.get('id'))
+        if term_id != self.term_id:
             return
         self._set_pty_size()
 
     def on_term_stdin(self, data):
-        if data.get('id') != self.term_id:
+        term_id = int(data.get('id'))
+        if term_id != self.term_id:
             return
         if not self.options.create:
             return
         self.handle_stdio(base64.b64decode(data['data']), data.get('user_id'))
 
     def on_term_stdout(self, data):
-        if data.get('id') != self.term_id:
+        term_id = int(data.get('id'))
+        if term_id != self.term_id:
             return
         self.handle_stdio(data['data'])
 
@@ -782,7 +797,11 @@ class Flootty(object):
             # reconnected. don't spawn a new shell
             out('Reconnected to %s' % (self.workspace_url()))
             return
-        shell = os.environ['SHELL']
+        shell = self.options.shell
+        if shell is None:
+            default_shell = '/bin/sh'
+            out('SHELL not set as an enviornment variable, unable to determine your default shell, using %s' % default_shell)
+            shell = default_shell
         out('Successfully joined %s' % (self.workspace_url()))
 
         self.child_pid, self.master_fd = pty.fork()
@@ -833,8 +852,12 @@ class Flootty(object):
             a = '\a'.encode('utf-8')
             n = '\n'.encode('utf-8')
             r = '\r'.encode('utf-8')
+            eof = '\004'.encode('utf-8')
             empty = ''.encode('utf-8')
             if self.options.safe:
+                # Stop people from using ctrl+d to end sessions
+                if buf.find(eof) != -1:
+                    buf = buf.replace(eof, empty)
                 if buf.find(n) != -1 or buf.find(r) != -1:
                     to = user_id or []
                     self.transport('datamsg', {
@@ -871,7 +894,10 @@ class Flootty(object):
                 term_color = color_green
             # Not confusing at all </sarcasm>
             cmd = 'PS1="%s%s::%s::%s%s%s%s $PS1"\n' % (color_green, self.owner, self.workspace, color_reset, term_color, self.term_name, color_reset)
-            write(self.master_fd, cmd)
+            if 'fish' in shell:
+                out('Unable to set prompt for your shell. Do not forget that you are sharing this terminal!')
+            else:
+                write(self.master_fd, cmd)
 
     def _signal_winch(self, signum, frame):
         '''
